function [xy, dist, filename] = generate_nodes(N, len, range, d_sigma, copyname, name);

if nargin > 4 && ~isempty(copyname)
    copy = load(copyname);
    xy = copy.xy;
else
    % random distribution of sensor nodes
    xy = unifrnd(0, len, N, 2);
end

dist = zeros(N,N);

for i = 1:N,
    for j = i+1:N,
        d = norm(xy(i,:)-xy(j,:));
        if (d <= range)
            noise = normrnd(0, d_sigma);
            dist(i,j) = d + noise;
            dist(j,i) = d + noise;
        end
    end
end

% Compute the average degree of node connectivity.
avg_degree = mean(sum(dist ~= 0));
fprintf('Average node degree is %f\n', avg_degree);

if nargin > 5
    filename = name;
else
    filename = get_unique_filename('nodes');
end

fprintf('Saved node layout in file "%s.mat"\n', filename);
save(filename, 'xy', 'dist');

fid = fopen([filename '.info'], 'w');
fprintf(fid, '%d nodes\n', N);
fprintf(fid, 'Region of length %f, comm range %f\n', len, range);
fprintf(fid, 'Std. dev. of measurement noise is %f\n', d_sigma);
fprintf(fid, 'Average node degree is %f\n', avg_degree);
fclose(fid);

generate_tossim_input(filename, dist);
